searchData={"content_type":"text/markdown","items":[{"doc":"A HTTP toolkit for building API clients using middlewares.","ref":"Tesla.html","title":"Tesla","type":"module"},{"doc":"`use Tesla` macro will generate basic HTTP functions (e.g. `get/3`, `post/4`, etc.) inside your module.\n\nIt supports following options:\n\n- `:only` - builder will generate only functions included in the given list\n- `:except` - builder will not generate the functions that are listed in the options\n- `:docs` - when set to false builder will not add documentation to generated functions","ref":"Tesla.html#module-building-api-client","title":"Building API client - Tesla","type":"module"},{"doc":"defmodule ExampleApi do\n      use Tesla, only: [:get], docs: false\n\n      plug Tesla.Middleware.BaseUrl, \"http://api.example.com\"\n      plug Tesla.Middleware.JSON\n\n      def fetch_data do\n        get(\"/data\")\n      end\n    end\n\nIn example above `ExampleApi.fetch_data/0` is equivalent of `ExampleApi.get(\"/data\")`.\n\n    defmodule ExampleApi do\n      use Tesla, except: [:post, :delete]\n\n      plug Tesla.Middleware.BaseUrl, \"http://api.example.com\"\n      plug Tesla.Middleware.JSON\n\n      def fetch_data do\n        get(\"/data\")\n      end\n    end\n\nIn example above `except: [:post, :delete]` will make sure that post functions will not be generated for this module.","ref":"Tesla.html#module-examples","title":"Examples - Tesla","type":"module"},{"doc":"It is also possible to do request directly with `Tesla` module.\n\n    Tesla.get(\"https://example.com\")","ref":"Tesla.html#module-direct-usage","title":"Direct usage - Tesla","type":"module"},{"doc":"Direct usage won't include any middlewares.\n\nIn following example:\n\n    defmodule ExampleApi do\n      use Tesla, only: [:get], docs: false\n\n      plug Tesla.Middleware.BaseUrl, \"http://api.example.com\"\n      plug Tesla.Middleware.JSON\n\n      def fetch_data do\n        Tesla.get(\"/data\")\n      end\n    end\n\ncall to `ExampleApi.fetch_data/0` will fail, because request will be missing base URL.","ref":"Tesla.html#module-common-pitfalls","title":"Common pitfalls - Tesla","type":"module"},{"doc":"By default `Tesla` is using `Tesla.Adapter.Httpc`, because `:httpc` is included in Erlang/OTP and\ndoes not require installation of any additional dependency. It can be changed globally with config:\n\n    config :tesla, :adapter, Tesla.Adapter.Hackney\n\nor by `Tesla.Builder.adapter/2` macro for given API client module:\n\n    defmodule ExampleApi do\n      use Tesla\n\n      adapter Tesla.Adapter.Hackney\n\n      ...\n    end","ref":"Tesla.html#module-default-adapter","title":"Default adapter - Tesla","type":"module"},{"doc":"","ref":"Tesla.html#build_adapter/1","title":"Tesla.build_adapter/1","type":"function"},{"doc":"","ref":"Tesla.html#build_client/2","title":"Tesla.build_client/2","type":"function"},{"doc":"Builds URL with the given query params.\n\nUseful when you need to create an URL with dynamic query params from a Keyword list","ref":"Tesla.html#build_url/2","title":"Tesla.build_url/2","type":"function"},{"doc":"iex> Tesla.build_url(\"http://api.example.com\", [user: 3, page: 2])\n    \"http://api.example.com?user=3&page=2\"\n\n    # URL that already contains query params\n    iex> url = \"http://api.example.com?user=3\"\n    iex> Tesla.build_url(url, [page: 2, status: true])\n    \"http://api.example.com?user=3&page=2&status=true\"","ref":"Tesla.html#build_url/2-examples","title":"Examples - Tesla.build_url/2","type":"function"},{"doc":"Dynamically build client from list of middlewares and/or adapter.\n\n```\n# add dynamic middleware\nclient = Tesla.client([{Tesla.Middleware.Headers, [{\"authorization\", token}]}])\nTesla.get(client, \"/path\")\n\n# configure adapter in runtime\nclient = Tesla.client([], Tesla.Adapter.Hackney)\nclient = Tesla.client([], {Tesla.Adapter.Hackney, pool: :my_pool})\nTesla.get(client, \"/path\")\n\n# complete module example\ndefmodule MyApi do\n  # note there is no need for `use Tesla`\n\n  @middleware [\n    {Tesla.Middleware.BaseUrl, \"https://example.com\"},\n    Tesla.Middleware.JSON,\n    Tesla.Middleware.Logger\n  ]\n\n  @adapter Tesla.Adapter.Hackney\n\n  def new(opts) do\n    # do any middleware manipulation you need\n    middleware = [\n      {Tesla.Middleware.BasicAuth, username: opts[:username], password: opts[:password]}\n    ] ++ @middleware\n\n    # allow configuring adapter in runtime\n    adapter = opts[:adapter] || @adapter\n\n    # use Tesla.client/2 to put it all together\n    Tesla.client(middleware, adapter)\n  end\n\n  def get_something(client, id) do\n    # pass client directly to Tesla.get/2\n    Tesla.get(client, \"/something/#{id}\")\n    # ...\n  end\nend\n\nclient = MyApi.new(username: \"admin\", password: \"secret\")\nMyApi.get_something(client, 42)\n```","ref":"Tesla.html#client/2","title":"Tesla.client/2","type":"function"},{"doc":"Perform a DELETE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    delete(\"/users\")\n    delete(\"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\")\n    delete(client, \"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#delete/3","title":"Tesla.delete/3","type":"function"},{"doc":"Perform a DELETE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    delete!(\"/users\")\n    delete!(\"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\")\n    delete!(client, \"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#delete!/3","title":"Tesla.delete!/3","type":"function"},{"doc":"","ref":"Tesla.html#delete_header/2","title":"Tesla.delete_header/2","type":"function"},{"doc":"","ref":"Tesla.html#encode_query/1","title":"Tesla.encode_query/1","type":"function"},{"doc":"Perform a GET request.\n\nSee `request/1` or `request/2` for options definition.\n\n    get(\"/users\")\n    get(\"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\")\n    get(client, \"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#get/3","title":"Tesla.get/3","type":"function"},{"doc":"Perform a GET request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    get!(\"/users\")\n    get!(\"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\")\n    get!(client, \"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#get!/3","title":"Tesla.get!/3","type":"function"},{"doc":"Returns value of header specified by `key` from `:headers` field in `Tesla.Env`.","ref":"Tesla.html#get_header/2","title":"Tesla.get_header/2","type":"function"},{"doc":"# non existing header\n    iex> env = %Tesla.Env{headers: [{\"server\", \"Cowboy\"}]}\n    iex> Tesla.get_header(env, \"some-key\")\n    nil\n\n    # existing header\n    iex> env = %Tesla.Env{headers: [{\"server\", \"Cowboy\"}]}\n    iex> Tesla.get_header(env, \"server\")\n    \"Cowboy\"\n\n    # first of multiple headers with the same name\n    iex> env = %Tesla.Env{headers: [{\"cookie\", \"chocolate\"}, {\"cookie\", \"biscuits\"}]}\n    iex> Tesla.get_header(env, \"cookie\")\n    \"chocolate\"","ref":"Tesla.html#get_header/2-examples","title":"Examples - Tesla.get_header/2","type":"function"},{"doc":"","ref":"Tesla.html#get_headers/2","title":"Tesla.get_headers/2","type":"function"},{"doc":"Perform a HEAD request.\n\nSee `request/1` or `request/2` for options definition.\n\n    head(\"/users\")\n    head(\"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\")\n    head(client, \"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#head/3","title":"Tesla.head/3","type":"function"},{"doc":"Perform a HEAD request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    head!(\"/users\")\n    head!(\"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\")\n    head!(client, \"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#head!/3","title":"Tesla.head!/3","type":"function"},{"doc":"Perform a OPTIONS request.\n\nSee `request/1` or `request/2` for options definition.\n\n    options(\"/users\")\n    options(\"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\")\n    options(client, \"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#options/3","title":"Tesla.options/3","type":"function"},{"doc":"Perform a OPTIONS request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    options!(\"/users\")\n    options!(\"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\")\n    options!(client, \"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#options!/3","title":"Tesla.options!/3","type":"function"},{"doc":"Perform a PATCH request.\n\nSee `request/1` or `request/2` for options definition.\n\n    patch(\"/users\", %{name: \"Jon\"})\n    patch(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch(client, \"/users\", %{name: \"Jon\"})\n    patch(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#patch/4","title":"Tesla.patch/4","type":"function"},{"doc":"Perform a PATCH request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    patch!(\"/users\", %{name: \"Jon\"})\n    patch!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch!(client, \"/users\", %{name: \"Jon\"})\n    patch!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#patch!/4","title":"Tesla.patch!/4","type":"function"},{"doc":"Perform a POST request.\n\nSee `request/1` or `request/2` for options definition.\n\n    post(\"/users\", %{name: \"Jon\"})\n    post(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post(client, \"/users\", %{name: \"Jon\"})\n    post(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#post/4","title":"Tesla.post/4","type":"function"},{"doc":"Perform a POST request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    post!(\"/users\", %{name: \"Jon\"})\n    post!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post!(client, \"/users\", %{name: \"Jon\"})\n    post!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#post!/4","title":"Tesla.post!/4","type":"function"},{"doc":"Perform a PUT request.\n\nSee `request/1` or `request/2` for options definition.\n\n    put(\"/users\", %{name: \"Jon\"})\n    put(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put(client, \"/users\", %{name: \"Jon\"})\n    put(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#put/4","title":"Tesla.put/4","type":"function"},{"doc":"Perform a PUT request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    put!(\"/users\", %{name: \"Jon\"})\n    put!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put!(client, \"/users\", %{name: \"Jon\"})\n    put!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#put!/4","title":"Tesla.put!/4","type":"function"},{"doc":"","ref":"Tesla.html#put_body/2","title":"Tesla.put_body/2","type":"function"},{"doc":"","ref":"Tesla.html#put_header/3","title":"Tesla.put_header/3","type":"function"},{"doc":"","ref":"Tesla.html#put_headers/2","title":"Tesla.put_headers/2","type":"function"},{"doc":"Adds given key/value pair to `:opts` field in `Tesla.Env`.\n\nUseful when there's need to store additional middleware data in `Tesla.Env`","ref":"Tesla.html#put_opt/3","title":"Tesla.put_opt/3","type":"function"},{"doc":"iex> %Tesla.Env{opts: []} |> Tesla.put_opt(:option, \"value\")\n    %Tesla.Env{opts: [option: \"value\"]}","ref":"Tesla.html#put_opt/3-examples","title":"Examples - Tesla.put_opt/3","type":"function"},{"doc":"Perform a request.","ref":"Tesla.html#request/2","title":"Tesla.request/2","type":"function"},{"doc":"- `:method` - the request method, one of [`:head`, `:get`, `:delete`, `:trace`, `:options`, `:post`, `:put`, `:patch`]\n- `:url` - either full url e.g. \"http://example.com/some/path\" or just \"/some/path\" if using `Tesla.Middleware.BaseUrl`\n- `:query` - a keyword list of query params, e.g. `[page: 1, per_page: 100]`\n- `:headers` - a keyworld list of headers, e.g. `[{\"content-type\", \"text/plain\"}]`\n- `:body` - depends on used middleware:\n    - by default it can be a binary\n    - if using e.g. JSON encoding middleware it can be a nested map\n    - if adapter supports it it can be a Stream with any of the above\n- `:opts` - custom, per-request middleware or adapter options","ref":"Tesla.html#request/2-options","title":"Options - Tesla.request/2","type":"function"},{"doc":"ExampleApi.request(method: :get, url: \"/users/path\")\n\n    # use shortcut methods\n    ExampleApi.get(\"/users/1\")\n    ExampleApi.post(client, \"/users\", %{name: \"Jon\"})","ref":"Tesla.html#request/2-examples","title":"Examples - Tesla.request/2","type":"function"},{"doc":"Perform request and raise in case of error.\n\nThis is similar to `request/2` behaviour from Tesla 0.x\n\nSee `request/2` for list of available options.","ref":"Tesla.html#request!/2","title":"Tesla.request!/2","type":"function"},{"doc":"","ref":"Tesla.html#run/2","title":"Tesla.run/2","type":"function"},{"doc":"","ref":"Tesla.html#run_default_adapter/2","title":"Tesla.run_default_adapter/2","type":"function"},{"doc":"Perform a TRACE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    trace(\"/users\")\n    trace(\"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\")\n    trace(client, \"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#trace/3","title":"Tesla.trace/3","type":"function"},{"doc":"Perform a TRACE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    trace!(\"/users\")\n    trace!(\"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\")\n    trace!(client, \"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#trace!/3","title":"Tesla.trace!/3","type":"function"},{"doc":"Options that may be passed to a request function. See `request/2` for detailed descriptions.","ref":"Tesla.html#t:option/0","title":"Tesla.option/0","type":"type"},{"doc":"","ref":"Tesla.Builder.html","title":"Tesla.Builder","type":"module"},{"doc":"","ref":"Tesla.Builder.html#adapter/1","title":"Tesla.Builder.adapter/1","type":"macro"},{"doc":"Choose adapter for your API client.\n\n```\ndefmodule ExampleApi do\n  use Tesla\n\n  # set adapter as module\n  adapter Tesla.Adapter.Hackney\n\n  # set adapter as anonymous function\n  adapter fn env ->\n    ...\n    env\n  end\nend","ref":"Tesla.Builder.html#adapter/2","title":"Tesla.Builder.adapter/2","type":"macro"},{"doc":"","ref":"Tesla.Builder.html#client/3","title":"Tesla.Builder.client/3","type":"function"},{"doc":"","ref":"Tesla.Builder.html#plug/1","title":"Tesla.Builder.plug/1","type":"macro"},{"doc":"Attach middleware to your API client.\n\n```elixir\ndefmodule ExampleApi do\n  use Tesla\n\n  # plug middleware module with options\n  plug Tesla.Middleware.BaseUrl, \"http://api.example.com\"\n\n  # or without options\n  plug Tesla.Middleware.JSON\n\n  # or a custom middleware\n  plug MyProject.CustomMiddleware\nend","ref":"Tesla.Builder.html#plug/2","title":"Tesla.Builder.plug/2","type":"macro"},{"doc":"","ref":"Tesla.Client.html","title":"Tesla.Client","type":"module"},{"doc":"Returns the client's adapter in the same form it was provided.\nThis can be used to copy an adapter from one client to another.","ref":"Tesla.Client.html#adapter/1","title":"Tesla.Client.adapter/1","type":"function"},{"doc":"iex> client = Tesla.client([], {Tesla.Adapter.Hackney, [recv_timeout: 30_000]})\n    iex> Tesla.Client.adapter(client)\n    {Tesla.Adapter.Hackney, [recv_timeout: 30_000]}","ref":"Tesla.Client.html#adapter/1-examples","title":"Examples - Tesla.Client.adapter/1","type":"function"},{"doc":"Returns the client's middleware in the same form it was provided.\nThis can be used to copy middleware from one client to another.","ref":"Tesla.Client.html#middleware/1","title":"Tesla.Client.middleware/1","type":"function"},{"doc":"iex> middleware = [Tesla.Middleware.JSON, {Tesla.Middleware.BaseUrl, \"https://api.github.com\"}]\n    iex> client = Tesla.client(middleware)\n    iex> Tesla.Client.middleware(client)\n    [Tesla.Middleware.JSON, {Tesla.Middleware.BaseUrl, \"https://api.github.com\"}]","ref":"Tesla.Client.html#middleware/1-examples","title":"Examples - Tesla.Client.middleware/1","type":"function"},{"doc":"","ref":"Tesla.Client.html#t:adapter/0","title":"Tesla.Client.adapter/0","type":"type"},{"doc":"","ref":"Tesla.Client.html#t:middleware/0","title":"Tesla.Client.middleware/0","type":"type"},{"doc":"","ref":"Tesla.Client.html#t:t/0","title":"Tesla.Client.t/0","type":"type"},{"doc":"This module defines a `t:Tesla.Env.t/0` struct that stores all data related to request/response.","ref":"Tesla.Env.html","title":"Tesla.Env","type":"module"},{"doc":"- `:method` - method of request. Example: `:get`\n- `:url` - request url. Example: `\"https://www.google.com\"`\n- `:query` - list of query params.\n  Example: `[{\"param\", \"value\"}]` will be translated to `?params=value`.\n  Note: query params passed in url (e.g. `\"/get?param=value\"`) are not parsed to `query` field.\n- `:headers` - list of request/response headers.\n  Example: `[{\"content-type\", \"application/json\"}]`.\n  Note: request headers are overridden by response headers when adapter is called.\n- `:body` - request/response body.\n  Note: request body is overridden by response body when adapter is called.\n- `:status` - response status. Example: `200`\n- `:opts` - list of options. Example: `[adapter: [recv_timeout: 30_000]]`","ref":"Tesla.Env.html#module-fields","title":"Fields - Tesla.Env","type":"module"},{"doc":"","ref":"Tesla.Env.html#t:body/0","title":"Tesla.Env.body/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:client/0","title":"Tesla.Env.client/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:headers/0","title":"Tesla.Env.headers/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:method/0","title":"Tesla.Env.method/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:opts/0","title":"Tesla.Env.opts/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:param/0","title":"Tesla.Env.param/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:query/0","title":"Tesla.Env.query/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:result/0","title":"Tesla.Env.result/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:runtime/0","title":"Tesla.Env.runtime/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:stack/0","title":"Tesla.Env.stack/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:status/0","title":"Tesla.Env.status/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:t/0","title":"Tesla.Env.t/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:url/0","title":"Tesla.Env.url/0","type":"type"},{"doc":"Mock adapter for better testing.","ref":"Tesla.Mock.html","title":"Tesla.Mock","type":"module"},{"doc":"```elixir\n# config/test.exs\nconfig :tesla, adapter: Tesla.Mock\n\n# in case MyClient defines specific adapter with `adapter SpecificAdapter`\nconfig :tesla, MyClient, adapter: Tesla.Mock\n```","ref":"Tesla.Mock.html#module-setup","title":"Setup - Tesla.Mock","type":"module"},{"doc":"```elixir\ndefmodule MyAppTest do\n  use ExUnit.Case\n\n  setup do\n    Tesla.Mock.mock(fn\n      %{method: :get} ->\n        %Tesla.Env{status: 200, body: \"hello\"}\n    end)\n\n    :ok\n  end\n\n  test \"list things\" do\n    assert {:ok, env} = MyApp.get(\"...\")\n    assert env.status == 200\n    assert env.body == \"hello\"\n  end\nend\n```","ref":"Tesla.Mock.html#module-examples","title":"Examples - Tesla.Mock","type":"module"},{"doc":"```elixir\n# Match on method & url and return whole Tesla.Env\nTesla.Mock.mock(fn\n  %{method: :get, url: \"http://example.com/list\"} ->\n    %Tesla.Env{status: 200, body: \"hello\"}\nend)\n\n# You can use any logic required\nTesla.Mock.mock(fn env ->\n  case env.url do\n    \"http://example.com/list\" ->\n      %Tesla.Env{status: 200, body: \"ok!\"}\n\n    _ ->\n      %Tesla.Env{status: 404, body: \"NotFound\"}\n  end\nend)\n\n\n# mock will also accept short version of response\n# in the form of {status, headers, body}\nTesla.Mock.mock(fn\n  %{method: :post} -> {201, %{}, %{id: 42}}\nend)\n\n# mock will also accept error tuples in the form\n# of {:error, reason}\nTesla.Mock.mock(fn\n  %{method: :post} -> {:error, :timeout}\nend)\n```","ref":"Tesla.Mock.html#module-setting-up-mocks","title":"Setting up mocks - Tesla.Mock","type":"module"},{"doc":"By default, mocks are bound to the current process,\ni.e. the process running a single test case.\nThis design allows proper isolation between test cases\nand make testing in parallel (`async: true`) possible.\n\nWhile this style is recommended, there is one drawback:\nif Tesla client is called from different process\nit will not use the setup mock.\n\nTo solve this issue it is possible to setup a global mock\nusing `mock_global/1` function.\n\n```elixir\ndefmodule MyTest do\n  use ExUnit.Case, async: false # must be false!\n\n  setup_all do\n    Tesla.Mock.mock_global fn\n      env -> # ...\n    end\n\n    :ok\n  end\n\n  # ...\nend\n```\n\n**WARNING**: Using global mocks may affect tests with local mock\n(because of fallback to global mock in case local one is not found)","ref":"Tesla.Mock.html#module-global-mocks","title":"Global mocks - Tesla.Mock","type":"module"},{"doc":"","ref":"Tesla.Mock.html#call/2","title":"Tesla.Mock.call/2","type":"function"},{"doc":"Return JSON response.\n\nExample\n\n    import Tesla.Mock\n\n    mock fn\n      %{url: \"/ok\"} -> json(%{\"some\" => \"data\"})\n      %{url: \"/404\"} -> json(%{\"some\" => \"data\"}, status: 404)\n    end","ref":"Tesla.Mock.html#json/2","title":"Tesla.Mock.json/2","type":"function"},{"doc":"Setup mocks for current test.\n\nThis mock will only be available to the current process.","ref":"Tesla.Mock.html#mock/1","title":"Tesla.Mock.mock/1","type":"function"},{"doc":"Setup global mocks.\n\n**WARNING**: This mock will be available to ALL processes.\nIt might cause conflicts when running tests in parallel!","ref":"Tesla.Mock.html#mock_global/1","title":"Tesla.Mock.mock_global/1","type":"function"},{"doc":"Return text response.\n\nExample\n\n    import Tesla.Mock\n\n    mock fn\n      %{url: \"/ok\"} -> text(%{\"some\" => \"data\"})\n      %{url: \"/404\"} -> text(%{\"some\" => \"data\"}, status: 404)\n    end","ref":"Tesla.Mock.html#text/2","title":"Tesla.Mock.text/2","type":"function"},{"doc":"","ref":"Tesla.Mock.html#t:response_opt/0","title":"Tesla.Mock.response_opt/0","type":"type"},{"doc":"","ref":"Tesla.Mock.html#t:response_opts/0","title":"Tesla.Mock.response_opts/0","type":"type"},{"doc":"Multipart functionality.","ref":"Tesla.Multipart.html","title":"Tesla.Multipart","type":"module"},{"doc":"```\nmp =\n  Multipart.new()\n  |> Multipart.add_content_type_param(\"charset=utf-8\")\n  |> Multipart.add_field(\"field1\", \"foo\")\n  |> Multipart.add_field(\"field2\", \"bar\",\n    headers: [{\"content-id\", \"1\"}, {\"content-type\", \"text/plain\"}]\n  )\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\")\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\", name: \"foobar\")\n  |> Multipart.add_file_content(\"sample file content\", \"sample.txt\")\n\nresponse = client.post(url, mp)\n```","ref":"Tesla.Multipart.html#module-examples","title":"Examples - Tesla.Multipart","type":"module"},{"doc":"Add a parameter to the multipart content-type.","ref":"Tesla.Multipart.html#add_content_type_param/2","title":"Tesla.Multipart.add_content_type_param/2","type":"function"},{"doc":"Add a field part.","ref":"Tesla.Multipart.html#add_field/4","title":"Tesla.Multipart.add_field/4","type":"function"},{"doc":"Add a file part. The file will be streamed.","ref":"Tesla.Multipart.html#add_file/3","title":"Tesla.Multipart.add_file/3","type":"function"},{"doc":"- `:name` - name of form param\n- `:filename` - filename (defaults to path basename)\n- `:headers` - additional headers\n- `:detect_content_type` - auto-detect file content-type (defaults to false)","ref":"Tesla.Multipart.html#add_file/3-options","title":"Options - Tesla.Multipart.add_file/3","type":"function"},{"doc":"Add a file part with value.\n\nSame of `add_file/3` but the file content is read from `data` input argument.","ref":"Tesla.Multipart.html#add_file_content/4","title":"Tesla.Multipart.add_file_content/4","type":"function"},{"doc":"- `:name` - name of form param\n- `:headers` - additional headers","ref":"Tesla.Multipart.html#add_file_content/4-options","title":"Options - Tesla.Multipart.add_file_content/4","type":"function"},{"doc":"Create a new Multipart struct to be used for a request body.","ref":"Tesla.Multipart.html#new/0","title":"Tesla.Multipart.new/0","type":"function"},{"doc":"","ref":"Tesla.Multipart.html#t:part_stream/0","title":"Tesla.Multipart.part_stream/0","type":"type"},{"doc":"","ref":"Tesla.Multipart.html#t:part_value/0","title":"Tesla.Multipart.part_value/0","type":"type"},{"doc":"","ref":"Tesla.Multipart.html#t:t/0","title":"Tesla.Multipart.t/0","type":"type"},{"doc":"The adapter specification.\n\nAdapter is a module that denormalize request data stored in `Tesla.Env` in order to make\nrequest with lower level http client (e.g. `:httpc` or `:hackney`) and normalize response data\nin order to store it back to `Tesla.Env`. It has to implement `c:Tesla.Adapter.call/2`.","ref":"Tesla.Adapter.html","title":"Tesla.Adapter","type":"behaviour"},{"doc":"Create a module implementing `c:Tesla.Adapter.call/2`.\n\nSee `c:Tesla.Adapter.call/2` for details.","ref":"Tesla.Adapter.html#module-writing-custom-adapter","title":"Writing custom adapter - Tesla.Adapter","type":"behaviour"},{"doc":"defmodule MyProject.CustomAdapter do\n      alias Tesla.Multipart\n\n      @behaviour Tesla.Adapter\n\n      @override_defaults [follow_redirect: false]\n\n      @impl Tesla.Adapter\n      def call(env, opts) do\n        opts = Tesla.Adapter.opts(@override_defaults, env, opts)\n\n        with {:ok, {status, headers, body}} <- request(env.method, env.body, env.headers, opts) do\n          {:ok, normalize_response(env, status, headers, body)}\n        end\n      end\n\n      defp request(_method, %Stream{}, _headers, _opts) do\n        {:error, \"stream not supported by adapter\"}\n      end\n\n      defp request(_method, %Multipart{}, _headers, _opts) do\n        {:error, \"multipart not supported by adapter\"}\n      end\n\n      defp request(method, body, headers, opts) do\n        :lower_level_http.request(method, body, denormalize_headers(headers), opts)\n      end\n\n      defp denormalize_headers(headers), do: ...\n      defp normalize_response(env, status, headers, body), do: %Tesla.Env{env | ...}\n    end","ref":"Tesla.Adapter.html#module-examples","title":"Examples - Tesla.Adapter","type":"behaviour"},{"doc":"Invoked when a request runs.","ref":"Tesla.Adapter.html#c:call/2","title":"Tesla.Adapter.call/2","type":"callback"},{"doc":"- `env` - `Tesla.Env` struct that stores request/response data\n- `options` - middleware options provided by user","ref":"Tesla.Adapter.html#c:call/2-arguments","title":"Arguments - Tesla.Adapter.call/2","type":"callback"},{"doc":"Helper function that merges all adapter options.","ref":"Tesla.Adapter.html#opts/3","title":"Tesla.Adapter.opts/3","type":"function"},{"doc":"- `defaults` (optional) - useful to override lower level http client default configuration\n- `env` - `Tesla.Env` struct\n- `opts` - options provided to `Tesla.Builder.adapter/2` macro","ref":"Tesla.Adapter.html#opts/3-arguments","title":"Arguments - Tesla.Adapter.opts/3","type":"function"},{"doc":"- config from `opts` overrides config from `defaults` when same key is encountered\n- config from `env` overrides config from both `defaults` and `opts` when same key is encountered","ref":"Tesla.Adapter.html#opts/3-precedence-rules","title":"Precedence rules - Tesla.Adapter.opts/3","type":"function"},{"doc":"The middleware specification.\n\nMiddleware is an extension of basic `Tesla` functionality. It is a module that must\nimplement `c:Tesla.Middleware.call/3`.","ref":"Tesla.Middleware.html","title":"Tesla.Middleware","type":"behaviour"},{"doc":"Options can be passed to middleware in second param of `Tesla.Builder.plug/2` macro:\n\n    plug Tesla.Middleware.BaseUrl, \"https://example.com\"\n\nor inside tuple in case of dynamic middleware (`Tesla.client/1`):\n\n    Tesla.client([{Tesla.Middleware.BaseUrl, \"https://example.com\"}])","ref":"Tesla.Middleware.html#module-middleware-options","title":"Middleware options - Tesla.Middleware","type":"behaviour"},{"doc":"The order in which middleware is defined matters. Note that the order when _sending_ the request\nmatches the order the middleware was defined in, but the order when _receiving_ the response\nis reversed.\n\nFor example, `Tesla.Middleware.DecompressResponse` must come _after_ `Tesla.Middleware.JSON`,\notherwise the response isn't decompressed before it reaches the JSON parser.","ref":"Tesla.Middleware.html#module-ordering","title":"Ordering - Tesla.Middleware","type":"behaviour"},{"doc":"Writing custom middleware is as simple as creating a module implementing `c:Tesla.Middleware.call/3`.\n\nSee `c:Tesla.Middleware.call/3` for details.","ref":"Tesla.Middleware.html#module-writing-custom-middleware","title":"Writing custom middleware - Tesla.Middleware","type":"behaviour"},{"doc":"defmodule MyProject.InspectHeadersMiddleware do\n      @behaviour Tesla.Middleware\n\n      @impl true\n      def call(env, next, _options) do\n        IO.inspect(env.headers)\n\n        with {:ok, env} <- Tesla.run(env, next) do\n          IO.inspect(env.headers)\n          {:ok, env}\n        end\n      end\n    end","ref":"Tesla.Middleware.html#module-examples","title":"Examples - Tesla.Middleware","type":"behaviour"},{"doc":"Invoked when a request runs.\n\n- (optionally) read and/or writes request data\n- calls `Tesla.run/2`\n- (optionally) read and/or writes response data","ref":"Tesla.Middleware.html#c:call/3","title":"Tesla.Middleware.call/3","type":"callback"},{"doc":"- `env` - `Tesla.Env` struct that stores request/response data\n- `next` - middlewares that should be called after current one\n- `options` - middleware options provided by user","ref":"Tesla.Middleware.html#c:call/3-arguments","title":"Arguments - Tesla.Middleware.call/3","type":"callback"},{"doc":"Adapter for [finch](https://github.com/sneako/finch).\n\nRemember to add `{:finch, \"~> 0.14.0\"}` to dependencies. Also, you need to\nrecompile tesla after adding the `:finch` dependency:\n\n```\nmix deps.clean tesla\nmix compile\n```","ref":"Tesla.Adapter.Finch.html","title":"Tesla.Adapter.Finch","type":"module"},{"doc":"In order to use Finch, you must start it and provide a `:name`. For example,\nin your supervision tree:\n\n```elixir\nchildren = [\n  {Finch, name: MyFinch}\n]\n```\n\nYou must provide the same name to this adapter:\n\n```\n# set globally in config/config.exs\nconfig :tesla, :adapter, {Tesla.Adapter.Finch, name: MyFinch}\n\n# set per module\ndefmodule MyClient do\n  use Tesla\n\n  adapter Tesla.Adapter.Finch, name: MyFinch\nend\n```","ref":"Tesla.Adapter.Finch.html#module-examples","title":"Examples - Tesla.Adapter.Finch","type":"module"},{"doc":"* `:name` - The `:name` provided to Finch (**required**).\n\n## [Finch options](https://hexdocs.pm/finch/Finch.html#request/3)\n\n  * `:pool_timeout` - This timeout is applied when a connection is checked\n    out from the pool. Default value is `5_000`.\n\n  * `:receive_timeout` - The maximum time to wait for a response before\n    returning an error. Default value is `15_000`.","ref":"Tesla.Adapter.Finch.html#module-adapter-specific-options","title":"Adapter specific options - Tesla.Adapter.Finch","type":"module"},{"doc":"Adapter for [gun](https://github.com/ninenines/gun).\n\nRemember to add `{:gun, \"~> 1.3\"}`, `{:idna, \"~> 6.0\"}` and `{:castore, \"~> 0.1\"}` to dependencies.\n\nIn version 1.3 gun sends `host` header with port. Fixed in master branch.\nAlso, you need to recompile tesla after adding `:gun` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Gun.html","title":"Tesla.Adapter.Gun","type":"module"},{"doc":"```\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Gun\n\n# set per module\ndefmodule MyClient do\n  use Tesla\n  adapter Tesla.Adapter.Gun\nend\n```","ref":"Tesla.Adapter.Gun.html#module-examples","title":"Examples - Tesla.Adapter.Gun","type":"module"},{"doc":"- `:timeout` - Time, while process, will wait for gun messages.\n\n- `:body_as` - What will be returned in `%Tesla.Env{}` body key. Possible values:\n    - `:plain` - as binary (default).\n    - `:stream` - as stream.\n        If you don't want to close connection (because you want to reuse it later)\n        pass `close_conn: false` in adapter opts.\n    - `:chunks` - as chunks.\n        You can get response body in chunks using `Tesla.Adapter.Gun.read_chunk/3` function.\n\n    Processing of the chunks and checking body size must be done by yourself.\n    Example of processing function is in `test/tesla/adapter/gun_test.exs` - `Tesla.Adapter.GunTest.read_body/4`.\n    If you don't need connection later don't forget to close it with `Tesla.Adapter.Gun.close/1`.\n\n- `:max_body` - Max response body size in bytes.\n    Works only with `body_as: :plain`, with other settings you need to check response body size by yourself.\n\n- `:conn` - Opened connection pid with gun. Is used for reusing gun connections.\n\n- `:close_conn` - Close connection or not after receiving full response body.\n    Is used for reusing gun connections. Defaults to `true`.\n\n- `:certificates_verification` - Add SSL certificates verification.\n    [erlang-certifi](https://github.com/certifi/erlang-certifi)\n    [ssl_verify_fun.erl](https://github.com/deadtrickster/ssl_verify_fun.erl)\n\n- `:proxy` - Proxy for requests.\n    **Socks proxy are supported from gun >= 2.0**.\n    Examples: `{'localhost', 1234}`, `{{127, 0, 0, 1}, 1234}`, `{:socks5, 'localhost', 1234}`.\n\n  **NOTE:** By default GUN uses TLS as transport if the specified port is 443,\n    if TLS is required for proxy connection on another port please specify transport\n    using the Gun options below otherwise tcp will be used.\n\n- `:proxy_auth` - Auth to be passed along with the proxy opt.\n    Supports Basic auth for regular and Socks proxy.\n    Format: `{proxy_username, proxy_password}`.\n\n## [Gun options](https://ninenines.eu/docs/en/gun/1.3/manual/gun/)\n\n- `:connect_timeout` - Connection timeout.\n\n- `:http_opts` - Options specific to the HTTP protocol.\n\n- `:http2_opts` -  Options specific to the HTTP/2 protocol.\n\n- `:protocols` - Ordered list of preferred protocols.\n    Defaults: `[:http2, :http]`- for :tls, `[:http]` - for :tcp.\n\n- `:trace` - Whether to enable dbg tracing of the connection process.\n    Should only be used during debugging. Default: false.\n\n- `:transport` - Whether to use TLS or plain TCP.\n    The default varies depending on the port used.\n    Port 443 defaults to tls. All other ports default to tcp.\n\n- `:transport_opts` - Transport options.\n    They are TCP options or TLS options depending on the selected transport.\n    Default: `[]`. Gun version: 1.3.\n\n- `:tls_opts` - TLS transport options.\n    Default: `[]`. Gun from master branch.\n\n- `:tcp_opts` - TCP transport options.\n    Default: `[]`. Gun from master branch.\n\n- `:socks_opts` - Options for socks.\n    Default: `[]`. Gun from master branch.\n\n- `:ws_opts` - Options specific to the Websocket protocol. Default: `%{}`.\n\n    - `:compress` - Whether to enable permessage-deflate compression.\n        This does not guarantee that compression will be used as it is the server\n        that ultimately decides. Defaults to false.\n\n    - `:protocols` - A non-empty list enables Websocket protocol negotiation.\n        The list of protocols will be sent in the sec-websocket-protocol request header.\n        The handler module interface is currently undocumented and must be set to `gun_ws_h`.","ref":"Tesla.Adapter.Gun.html#module-adapter-specific-options","title":"Adapter specific options - Tesla.Adapter.Gun","type":"module"},{"doc":"Brutally close the `gun` connection.","ref":"Tesla.Adapter.Gun.html#close/1","title":"Tesla.Adapter.Gun.close/1","type":"function"},{"doc":"Reads chunk of the response body.\n\nReturns `{:fin, binary()}` if all body received, otherwise returns `{:nofin, binary()}`.","ref":"Tesla.Adapter.Gun.html#read_chunk/3","title":"Tesla.Adapter.Gun.read_chunk/3","type":"function"},{"doc":"Adapter for [hackney](https://github.com/benoitc/hackney).\n\nRemember to add `{:hackney, \"~> 1.13\"}` to dependencies (and `:hackney` to applications in `mix.exs`)\nAlso, you need to recompile tesla after adding `:hackney` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Hackney.html","title":"Tesla.Adapter.Hackney","type":"module"},{"doc":"```\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Hackney\n\n# set per module\ndefmodule MyClient do\n  use Tesla\n\n  adapter Tesla.Adapter.Hackney\nend\n```","ref":"Tesla.Adapter.Hackney.html#module-examples","title":"Examples - Tesla.Adapter.Hackney","type":"module"},{"doc":"- `:max_body` - Max response body size in bytes. Actual response may be bigger because hackney stops after the last chunk that surpasses `:max_body`.","ref":"Tesla.Adapter.Hackney.html#module-adapter-specific-options","title":"Adapter specific options - Tesla.Adapter.Hackney","type":"module"},{"doc":"Adapter for [httpc](http://erlang.org/doc/man/httpc.html).\n\nThis is the default adapter.\n\n**NOTE** Tesla overrides default autoredirect value with false to ensure\nconsistency between adapters","ref":"Tesla.Adapter.Httpc.html","title":"Tesla.Adapter.Httpc","type":"module"},{"doc":"Adapter for [ibrowse](https://github.com/cmullaparthi/ibrowse).\n\nRemember to add `{:ibrowse, \"~> 4.2\"}` to dependencies (and `:ibrowse` to applications in `mix.exs`)\nAlso, you need to recompile tesla after adding `:ibrowse` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Ibrowse.html","title":"Tesla.Adapter.Ibrowse","type":"module"},{"doc":"```\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Ibrowse\n\n# set per module\ndefmodule MyClient do\n  use Tesla\n\n  adapter Tesla.Adapter.Ibrowse\nend\n```","ref":"Tesla.Adapter.Ibrowse.html#module-examples","title":"Examples - Tesla.Adapter.Ibrowse","type":"module"},{"doc":"Adapter for [mint](https://github.com/elixir-mint/mint).\n\n**NOTE:** The minimum supported Elixir version for mint is 1.5.0\n\nRemember to add `{:mint, \"~> 1.0\"}` and `{:castore, \"~> 0.1\"}` to dependencies.\nAlso, you need to recompile tesla after adding `:mint` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Mint.html","title":"Tesla.Adapter.Mint","type":"module"},{"doc":"```\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Mint\n# set per module\ndefmodule MyClient do\n  use Tesla\n  adapter Tesla.Adapter.Mint\nend\n\n# set global custom cacertfile\nconfig :tesla, adapter: {Tesla.Adapter.Mint, cacert: [\"path_to_cacert\"]}\n```","ref":"Tesla.Adapter.Mint.html#module-examples","title":"Examples - Tesla.Adapter.Mint","type":"module"},{"doc":"- `:timeout` - Time in milliseconds, while process, will wait for mint messages. Defaults to `2_000`.\n- `:body_as` - What will be returned in `%Tesla.Env{}` body key. Possible values - `:plain`, `:stream`, `:chunks`. Defaults to `:plain`.\n  - `:plain` - as binary.\n  - `:stream` - as stream. If you don't want to close connection (because you want to reuse it later) pass `close_conn: false` in adapter opts.\n  - `:chunks` - as chunks. You can get response body in chunks using `Tesla.Adapter.Mint.read_chunk/3` function.\n  Processing of the chunks and checking body size must be done by yourself. Example of processing function is in `test/tesla/adapter/mint_test.exs` - `Tesla.Adapter.MintTest.read_body/4`. If you don't need connection later don't forget to close it with `Tesla.Adapter.Mint.close/1`.\n- `:max_body` - Max response body size in bytes. Works only with `body_as: :plain`, with other settings you need to check response body size by yourself.\n- `:conn` - Opened connection with mint. Is used for reusing mint connections.\n- `:original` - Original host with port, for which reused connection was open. Needed for `Tesla.Middleware.FollowRedirects`. Otherwise adapter will use connection for another open host.\n- `:close_conn` - Close connection or not after receiving full response body. Is used for reusing mint connections. Defaults to `true`.\n- `:proxy` - Proxy settings. E.g.: `{:http, \"localhost\", 8888, []}`, `{:http, \"127.0.0.1\", 8888, []}`\n- `:transport_opts` - Keyword list of HTTP or HTTPS options passed into `:gen_tcp` or `:ssl` respectively by mint. See [mint's docs on `transport_opts`](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-transport-options).","ref":"Tesla.Adapter.Mint.html#module-adapter-specific-options","title":"Adapter specific options: - Tesla.Adapter.Mint","type":"module"},{"doc":"Closes mint connection.","ref":"Tesla.Adapter.Mint.html#close/1","title":"Tesla.Adapter.Mint.close/1","type":"function"},{"doc":"Reads chunk of the response body.\nReturns `{:fin, HTTP.t(), binary()}` if all body received, otherwise returns `{:nofin, HTTP.t(), binary()}`.","ref":"Tesla.Adapter.Mint.html#read_chunk/3","title":"Tesla.Adapter.Mint.read_chunk/3","type":"function"},{"doc":"Set base URL for all requests.\n\nThe base URL will be prepended to request path/URL only\nif it does not include http(s).","ref":"Tesla.Middleware.BaseUrl.html","title":"Tesla.Middleware.BaseUrl","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.BaseUrl, \"https://example.com/foo\"\nend\n\nMyClient.get(\"/path\") # equals to GET https://example.com/foo/path\nMyClient.get(\"path\") # equals to GET https://example.com/foo/path\nMyClient.get(\"\") # equals to GET https://example.com/foo\nMyClient.get(\"http://example.com/bar\") # equals to GET http://example.com/bar\n```","ref":"Tesla.Middleware.BaseUrl.html#module-examples","title":"Examples - Tesla.Middleware.BaseUrl","type":"module"},{"doc":"Basic authentication middleware.\n\n[Wiki on the topic](https://en.wikipedia.org/wiki/Basic_access_authentication)","ref":"Tesla.Middleware.BasicAuth.html","title":"Tesla.Middleware.BasicAuth","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  # static configuration\n  plug Tesla.Middleware.BasicAuth, username: \"user\", password: \"pass\"\n\n  # dynamic user & pass\n  def new(username, password, opts \\\\ %{}) do\n    Tesla.client [\n      {Tesla.Middleware.BasicAuth, Map.merge(%{username: username, password: password}, opts)}\n    ]\n  end\nend\n```","ref":"Tesla.Middleware.BasicAuth.html#module-examples","title":"Examples - Tesla.Middleware.BasicAuth","type":"module"},{"doc":"- `:username` - username (defaults to `\"\"`)\n- `:password` - password (defaults to `\"\"`)","ref":"Tesla.Middleware.BasicAuth.html#module-options","title":"Options - Tesla.Middleware.BasicAuth","type":"module"},{"doc":"Bearer authentication middleware.\n\nAdds a `{\"authorization\", \"Bearer  \"}` header.","ref":"Tesla.Middleware.BearerAuth.html","title":"Tesla.Middleware.BearerAuth","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  # static configuration\n  plug Tesla.Middleware.BearerAuth, token: \"token\"\n\n  # dynamic token\n  def new(token) do\n    Tesla.client [\n      {Tesla.Middleware.BearerAuth, token: token}\n    ]\n  end\nend\n```","ref":"Tesla.Middleware.BearerAuth.html#module-examples","title":"Examples - Tesla.Middleware.BearerAuth","type":"module"},{"doc":"- `:token` - token (defaults to `\"\"`)","ref":"Tesla.Middleware.BearerAuth.html#module-options","title":"Options - Tesla.Middleware.BearerAuth","type":"module"},{"doc":"Only compress request.\n\nSee `Tesla.Middleware.Compression` for options.","ref":"Tesla.Middleware.CompressRequest.html","title":"Tesla.Middleware.CompressRequest","type":"module"},{"doc":"Compress requests and decompress responses.\n\nSupports \"gzip\" and \"deflate\" encodings using Erlang's built-in `:zlib` module.","ref":"Tesla.Middleware.Compression.html","title":"Tesla.Middleware.Compression","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.Compression, format: \"gzip\"\nend\n```","ref":"Tesla.Middleware.Compression.html#module-examples","title":"Examples - Tesla.Middleware.Compression","type":"module"},{"doc":"- `:format` - request compression format, `\"gzip\"` (default) or `\"deflate\"`","ref":"Tesla.Middleware.Compression.html#module-options","title":"Options - Tesla.Middleware.Compression","type":"module"},{"doc":"Compress request.\n\nIt is used by `Tesla.Middleware.CompressRequest`.","ref":"Tesla.Middleware.Compression.html#compress/2","title":"Tesla.Middleware.Compression.compress/2","type":"function"},{"doc":"Decompress response.\n\nIt is used by `Tesla.Middleware.DecompressResponse`.","ref":"Tesla.Middleware.Compression.html#decompress/1","title":"Tesla.Middleware.Compression.decompress/1","type":"function"},{"doc":"","ref":"Tesla.Middleware.DecodeFormUrlencoded.html","title":"Tesla.Middleware.DecodeFormUrlencoded","type":"module"},{"doc":"Decodes response body as JSON.\n\nOnly decodes the body if the `Content-Type` header suggests\nthat the body is JSON.","ref":"Tesla.Middleware.DecodeJson.html","title":"Tesla.Middleware.DecodeJson","type":"module"},{"doc":"Decode `Link` Hypermedia HTTP header into `opts[:rels]` field in response.","ref":"Tesla.Middleware.DecodeRels.html","title":"Tesla.Middleware.DecodeRels","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.DecodeRels\nend\n\nenv = MyClient.get(\"/...\")\n\nenv.opts[:rels]\n# => %{\"Next\" => \"http://...\", \"Prev\" => \"...\"}\n```","ref":"Tesla.Middleware.DecodeRels.html#module-examples","title":"Examples - Tesla.Middleware.DecodeRels","type":"module"},{"doc":"Only decompress response.\n\nSee `Tesla.Middleware.Compression` for options.","ref":"Tesla.Middleware.DecompressResponse.html","title":"Tesla.Middleware.DecompressResponse","type":"module"},{"doc":"Digest access authentication middleware.\n\n[Wiki on the topic](https://en.wikipedia.org/wiki/Digest_access_authentication)\n\n**NOTE**: Currently the implementation is incomplete and works only for MD5 algorithm\nand auth \"quality of protection\" (qop).","ref":"Tesla.Middleware.DigestAuth.html","title":"Tesla.Middleware.DigestAuth","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  def client(username, password, opts \\ %{}) do\n    Tesla.client([\n      {Tesla.Middleware.DigestAuth, Map.merge(%{username: username, password: password}, opts)}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.DigestAuth.html#module-examples","title":"Examples - Tesla.Middleware.DigestAuth","type":"module"},{"doc":"- `:username` - username (defaults to `\"\"`)\n- `:password` - password (defaults to `\"\"`)\n- `:cnonce_fn` - custom function generating client nonce (defaults to `&Tesla.Middleware.DigestAuth.cnonce/0`)\n- `:nc` - nonce counter (defaults to `\"00000000\"`)","ref":"Tesla.Middleware.DigestAuth.html#module-options","title":"Options - Tesla.Middleware.DigestAuth","type":"module"},{"doc":"","ref":"Tesla.Middleware.EncodeFormUrlencoded.html","title":"Tesla.Middleware.EncodeFormUrlencoded","type":"module"},{"doc":"Encodes request body as JSON.","ref":"Tesla.Middleware.EncodeJson.html","title":"Tesla.Middleware.EncodeJson","type":"module"},{"doc":"Follow HTTP 3xx redirects.","ref":"Tesla.Middleware.FollowRedirects.html","title":"Tesla.Middleware.FollowRedirects","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.FollowRedirects, max_redirects: 3 # defaults to 5\nend\n```","ref":"Tesla.Middleware.FollowRedirects.html#module-examples","title":"Examples - Tesla.Middleware.FollowRedirects","type":"module"},{"doc":"- `:max_redirects` - limit number of redirects (default: `5`)","ref":"Tesla.Middleware.FollowRedirects.html#module-options","title":"Options - Tesla.Middleware.FollowRedirects","type":"module"},{"doc":"Send request body as `application/x-www-form-urlencoded`.\n\nPerforms encoding of `body` from a `Map` such as `%{\"foo\" => \"bar\"}` into\nurl encoded data.\n\nPerforms decoding of the response into a map when urlencoded and content-type\nis `application/x-www-form-urlencoded`, so `\"foo=bar\"` becomes\n`%{\"foo\" => \"bar\"}`.","ref":"Tesla.Middleware.FormUrlencoded.html","title":"Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"```\ndefmodule Myclient do\n  use Tesla\n\n  plug Tesla.Middleware.FormUrlencoded\nend\n\nMyclient.post(\"/url\", %{key: :value})\n```","ref":"Tesla.Middleware.FormUrlencoded.html#module-examples","title":"Examples - Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"- `:decode` - decoding function, defaults to `URI.decode_query/1`\n- `:encode` - encoding function, defaults to `URI.encode_query/1`","ref":"Tesla.Middleware.FormUrlencoded.html#module-options","title":"Options - Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"Natively, nested maps are not supported in the body, so\n`%{\"foo\" => %{\"bar\" => \"baz\"}}` won't be encoded and raise an error.\nSupport for this specific case is obtained by configuring the middleware to\nencode (and decode) with `Plug.Conn.Query`\n\n```\ndefmodule Myclient do\n  use Tesla\n\n  plug Tesla.Middleware.FormUrlencoded,\n    encode: &Plug.Conn.Query.encode/1,\n    decode: &Plug.Conn.Query.decode/1\nend\n\nMyclient.post(\"/url\", %{key: %{nested: \"value\"}})\n```","ref":"Tesla.Middleware.FormUrlencoded.html#module-nested-maps","title":"Nested Maps - Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"Decode response body as querystring.\n\nIt is used by `Tesla.Middleware.DecodeFormUrlencoded`.","ref":"Tesla.Middleware.FormUrlencoded.html#decode/2","title":"Tesla.Middleware.FormUrlencoded.decode/2","type":"function"},{"doc":"Encode response body as querystring.\n\nIt is used by `Tesla.Middleware.EncodeFormUrlencoded`.","ref":"Tesla.Middleware.FormUrlencoded.html#encode/2","title":"Tesla.Middleware.FormUrlencoded.encode/2","type":"function"},{"doc":"Circuit Breaker middleware using [fuse](https://github.com/jlouis/fuse).\n\nRemember to add `{:fuse, \"~> 2.4\"}` to dependencies (and `:fuse` to applications in `mix.exs`)\nAlso, you need to recompile tesla after adding `:fuse` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Middleware.Fuse.html","title":"Tesla.Middleware.Fuse","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.Fuse,\n    opts: {{:standard, 2, 10_000}, {:reset, 60_000}},\n    keep_original_error: true,\n    should_melt: fn\n      {:ok, %{status: status}} when status in [428, 500, 504] -> true\n      {:ok, _} -> false\n      {:error, _} -> true\n    end,\n    mode: :sync\nend\n```","ref":"Tesla.Middleware.Fuse.html#module-examples","title":"Examples - Tesla.Middleware.Fuse","type":"module"},{"doc":"- `:name` - fuse name (defaults to module name)\n- `:opts` - fuse options (see fuse docs for reference)\n- `:keep_original_error` - boolean to indicate if, in case of melting (based on `should_melt`), it should return the upstream's error or the fixed one `{:error, unavailable}`.\nIt's false by default, but it will be true in `2.0.0` version\n- `:should_melt` - function to determine if response should melt the fuse\n- `:mode` - how to query the fuse, which has two values:\n  - `:sync` - queries are serialized through the `:fuse_server` process (the default)\n  - `:async_dirty` - queries check the fuse state directly, but may not account for recent melts or resets","ref":"Tesla.Middleware.Fuse.html#module-options","title":"Options - Tesla.Middleware.Fuse","type":"module"},{"doc":"fuse library uses [SASL (System Architecture Support Libraries)](http://erlang.org/doc/man/sasl_app.html).\n\nYou can disable its logger output using:\n\n```\nconfig :sasl, sasl_error_logger: :false\n```\n\nRead more at [jlouis/fuse#32](https://github.com/jlouis/fuse/issues/32) and [jlouis/fuse#19](https://github.com/jlouis/fuse/issues/19).","ref":"Tesla.Middleware.Fuse.html#module-sasl-logger","title":"SASL logger - Tesla.Middleware.Fuse","type":"module"},{"doc":"Set default headers for all requests","ref":"Tesla.Middleware.Headers.html","title":"Tesla.Middleware.Headers","type":"module"},{"doc":"```\ndefmodule Myclient do\n  use Tesla\n\n  plug Tesla.Middleware.Headers, [{\"user-agent\", \"Tesla\"}]\nend\n```","ref":"Tesla.Middleware.Headers.html#module-examples","title":"Examples - Tesla.Middleware.Headers","type":"module"},{"doc":"Encode requests and decode responses as JSON.\n\nThis middleware requires [Jason](https://hex.pm/packages/jason) (or other engine) as dependency.\n\nRemember to add `{:jason, \">= 1.0\"}` to dependencies.\nAlso, you need to recompile Tesla after adding `:jason` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```\n\nIf you only need to encode the request body or decode the response body,\nyou can use `Tesla.Middleware.EncodeJson` or `Tesla.Middleware.DecodeJson` directly instead.","ref":"Tesla.Middleware.JSON.html","title":"Tesla.Middleware.JSON","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.JSON # use jason engine\n  # or\n  plug Tesla.Middleware.JSON, engine: JSX, engine_opts: [strict: [:comments]]\n  # or\n  plug Tesla.Middleware.JSON, engine: Poison, engine_opts: [keys: :atoms]\n  # or\n  plug Tesla.Middleware.JSON, decode: &JSX.decode/1, encode: &JSX.encode/1\nend\n```","ref":"Tesla.Middleware.JSON.html#module-examples","title":"Examples - Tesla.Middleware.JSON","type":"module"},{"doc":"- `:decode` - decoding function\n- `:encode` - encoding function\n- `:encode_content_type` - content-type to be used in request header\n- `:engine` - encode/decode engine, e.g `Jason`, `Poison` or `JSX`  (defaults to Jason)\n- `:engine_opts` - optional engine options\n- `:decode_content_types` - list of additional decodable content-types","ref":"Tesla.Middleware.JSON.html#module-options","title":"Options - Tesla.Middleware.JSON","type":"module"},{"doc":"Decode response body as JSON.\n\nIt is used by `Tesla.Middleware.DecodeJson`.","ref":"Tesla.Middleware.JSON.html#decode/2","title":"Tesla.Middleware.JSON.decode/2","type":"function"},{"doc":"Encode request body as JSON.\n\nIt is used by `Tesla.Middleware.EncodeJson`.","ref":"Tesla.Middleware.JSON.html#encode/2","title":"Tesla.Middleware.JSON.encode/2","type":"function"},{"doc":"Store request url ,body and headers into `:opts`.","ref":"Tesla.Middleware.KeepRequest.html","title":"Tesla.Middleware.KeepRequest","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.KeepRequest\n  plug Tesla.Middleware.PathParams\nend\n\n{:ok, env} = MyClient.post(\"/users/:user_id\", \"request-data\", opts: [path_params: [user_id: \"1234]])\n\nenv.body\n# => \"response-data\"\n\nenv.opts[:req_body]\n# => \"request-data\"\n\nenv.opts[:req_headers]\n# => [{\"request-headers\", \"are-safe\"}, ...]\n\nenv.opts[:req_url]\n# => \"http://localhost:8000/users/:user_id\n```","ref":"Tesla.Middleware.KeepRequest.html#module-examples","title":"Examples - Tesla.Middleware.KeepRequest","type":"module"},{"doc":"Log requests using Elixir's Logger.\n\nWith the default settings it logs request method, URL, response status, and time taken in milliseconds.","ref":"Tesla.Middleware.Logger.html","title":"Tesla.Middleware.Logger","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.Logger\nend\n```","ref":"Tesla.Middleware.Logger.html#module-examples","title":"Examples - Tesla.Middleware.Logger","type":"module"},{"doc":"- `:log_level` - custom function for calculating log level (see below)\n- `:filter_headers` - sanitizes sensitive headers before logging in debug mode (see below)\n- `:debug` - show detailed request/response logging\n- `:format` - custom string template or function for log message (see below)","ref":"Tesla.Middleware.Logger.html#module-options","title":"Options - Tesla.Middleware.Logger","type":"module"},{"doc":"The default log format is `\"$method $url -> $status ($time ms)\"`\nwhich shows in logs like:\n\n```\n2018-03-25 18:32:40.397 [info]  GET https://bitebot.io -> 200 (88.074 ms)\n```\n\nIt can be changed globally with config:\n\n```elixir\nconfig :tesla, Tesla.Middleware.Logger, format: \"$method $url ====> $status / time=$time\"\n```\n\nOr you can customize this setting by providing your own `format` function:\n\n```elixir\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.Logger, format: &my_format/3\n\n  def my_format(request, response, time) do\n    \"request=#{inspect(request)} response=#{inspect(response)} time=#{time}\\n\"\n  end\nend\n```","ref":"Tesla.Middleware.Logger.html#module-custom-log-format","title":"Custom log format - Tesla.Middleware.Logger","type":"module"},{"doc":"By default, the following log levels will be used:\n\n- `:error` - for errors, 5xx and 4xx responses\n- `:warn` or `:warning` - for 3xx responses\n- `:info` - for 2xx responses\n\nYou can customize this setting by providing your own `log_level/1` function:\n\n```elixir\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.Logger, log_level: &my_log_level/1\n\n  def my_log_level(env) do\n    case env.status do\n      404 -> :info\n      _ -> :default\n    end\n  end\nend\n```","ref":"Tesla.Middleware.Logger.html#module-custom-log-levels","title":"Custom log levels - Tesla.Middleware.Logger","type":"module"},{"doc":"When the Elixir Logger log level is set to `:debug`\nTesla Logger will show full request & response.\n\nIf you want to disable detailed request/response logging\nbut keep the `:debug` log level (i.e. in development)\nyou can set `debug: false` in your config:\n\n```\n# config/dev.local.exs\nconfig :tesla, Tesla.Middleware.Logger, debug: false\n```\n\nNote that the logging configuration is evaluated at compile time,\nso Tesla must be recompiled for the configuration to take effect:\n\n```\nmix deps.clean --build tesla\nmix deps.compile tesla\n```\n\nIn order to be able to set `:debug` at runtime we can\npass it as a option to the middleware at runtime.\n\n```elixir\ndef client do\n  middleware = [\n    # ...\n    {Tesla.Middleware.Logger, debug: false}\n  ]\n\n  Tesla.client(middleware)\nend\n```","ref":"Tesla.Middleware.Logger.html#module-logger-debug-output","title":"Logger Debug output - Tesla.Middleware.Logger","type":"module"},{"doc":"To sanitize sensitive headers such as `authorization` in\ndebug logs, add them to the `:filter_headers` option.\n`:filter_headers` expects a list of header names as strings.\n\n```\n# config/dev.local.exs\nconfig :tesla, Tesla.Middleware.Logger,\n  filter_headers: [\"authorization\"]\n```","ref":"Tesla.Middleware.Logger.html#module-filter-headers","title":"Filter headers - Tesla.Middleware.Logger","type":"module"},{"doc":"","ref":"Tesla.Middleware.Logger.html#default_log_level/1","title":"Tesla.Middleware.Logger.default_log_level/1","type":"function"},{"doc":"","ref":"Tesla.Middleware.Logger.html#t:log_level/0","title":"Tesla.Middleware.Logger.log_level/0","type":"type"},{"doc":"Encode requests and decode responses as MessagePack.\n\nThis middleware requires [Msgpax](https://hex.pm/packages/msgpax) as dependency.\n\nRemember to add `{:msgpax, \">= 2.3.0\"}` to dependencies.\nAlso, you need to recompile Tesla after adding `:msgpax` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Middleware.MessagePack.html","title":"Tesla.Middleware.MessagePack","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.MessagePack\n  # or\n  plug Tesla.Middleware.MessagePack, engine_opts: [binary: true]\n  # or\n  plug Tesla.Middleware.MessagePack, decode: &Custom.decode/1, encode: &Custom.encode/1\nend\n```","ref":"Tesla.Middleware.MessagePack.html#module-examples","title":"Examples - Tesla.Middleware.MessagePack","type":"module"},{"doc":"- `:decode` - decoding function\n- `:encode` - encoding function\n- `:encode_content_type` - content-type to be used in request header\n- `:decode_content_types` - list of additional decodable content-types\n- `:engine_opts` - optional engine options","ref":"Tesla.Middleware.MessagePack.html#module-options","title":"Options - Tesla.Middleware.MessagePack","type":"module"},{"doc":"Decode response body as MessagePack.\n\nIt is used by `Tesla.Middleware.DecodeMessagePack`.","ref":"Tesla.Middleware.MessagePack.html#decode/2","title":"Tesla.Middleware.MessagePack.decode/2","type":"function"},{"doc":"Encode request body as MessagePack.\n\nIt is used by `Tesla.Middleware.EncodeMessagePack`.","ref":"Tesla.Middleware.MessagePack.html#encode/2","title":"Tesla.Middleware.MessagePack.encode/2","type":"function"},{"doc":"Middleware that adds `X-HTTP-Method-Override` header with original request\nmethod and sends the request as post.\n\nUseful when there's an issue with sending non-POST request.","ref":"Tesla.Middleware.MethodOverride.html","title":"Tesla.Middleware.MethodOverride","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.MethodOverride\nend\n```","ref":"Tesla.Middleware.MethodOverride.html#module-examples","title":"Examples - Tesla.Middleware.MethodOverride","type":"module"},{"doc":"- `:override` - list of HTTP methods that should be overridden, everything except `:get` and `:post` if not specified","ref":"Tesla.Middleware.MethodOverride.html#module-options","title":"Options - Tesla.Middleware.MethodOverride","type":"module"},{"doc":"Set default opts for all requests.","ref":"Tesla.Middleware.Opts.html","title":"Tesla.Middleware.Opts","type":"module"},{"doc":"```\ndefmodule Myclient do\n  use Tesla\n\n  plug Tesla.Middleware.Opts, [some: \"option\"]\nend\n```","ref":"Tesla.Middleware.Opts.html#module-examples","title":"Examples - Tesla.Middleware.Opts","type":"module"},{"doc":"Use templated URLs with provided parameters in either Phoenix style (`:id`)\nor OpenAPI style (`{id}`).\n\nUseful when logging or reporting metrics per URL.","ref":"Tesla.Middleware.PathParams.html","title":"Tesla.Middleware.PathParams","type":"module"},{"doc":"Parameter values may be `t:struct/0` or must implement the `Enumerable`\nprotocol and produce `{key, value}` tuples when enumerated.","ref":"Tesla.Middleware.PathParams.html#module-parameter-values","title":"Parameter Values - Tesla.Middleware.PathParams","type":"module"},{"doc":"Phoenix style parameters may contain letters, numbers, or underscores,\nmatching this regular expression:\n\n  :[a-zA-Z][_a-zA-Z0-9]*\b\n\nOpenAPI style parameters may contain letters, numbers, underscores, or\nhyphens (`-`), matching this regular expression:\n\n  {[a-zA-Z][-_a-zA-Z0-9]*}\n\nIn either case, parameters that begin with underscores (`_`), hyphens (`-`),\nor numbers (`0-9`) are ignored and left as-is.","ref":"Tesla.Middleware.PathParams.html#module-parameter-name-restrictions","title":"Parameter Name Restrictions - Tesla.Middleware.PathParams","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.BaseUrl, \"https://api.example.com\"\n  plug Tesla.Middleware.Logger # or some monitoring middleware\n  plug Tesla.Middleware.PathParams\n\n  def user(id) do\n    params = [id: id]\n    get(\"/users/{id}\", opts: [path_params: params])\n  end\n\n  def posts(id, post_id) do\n    params = [id: id, post_id: post_id]\n    get(\"/users/:id/posts/:post_id\", opts: [path_params: params])\n  end\nend\n```","ref":"Tesla.Middleware.PathParams.html#module-examples","title":"Examples - Tesla.Middleware.PathParams","type":"module"},{"doc":"Set default query params for all requests","ref":"Tesla.Middleware.Query.html","title":"Tesla.Middleware.Query","type":"module"},{"doc":"```\ndefmodule Myclient do\n  use Tesla\n\n  plug Tesla.Middleware.Query, [token: \"some-token\"]\nend\n```","ref":"Tesla.Middleware.Query.html#module-examples","title":"Examples - Tesla.Middleware.Query","type":"module"},{"doc":"Retry using exponential backoff and full jitter.\n\nBy defaults, this middleware only retries in the case of connection errors (`nxdomain`, `connrefused`, etc).\nApplication error checking for retry can be customized through `:should_retry` option.","ref":"Tesla.Middleware.Retry.html","title":"Tesla.Middleware.Retry","type":"module"},{"doc":"The backoff algorithm optimizes for tight bounds on completing a request successfully.\nIt does this by first calculating an exponential backoff factor based on the\nnumber of retries that have been performed.  It then multiplies this factor against\nthe base delay. The total maximum delay is found by taking the minimum of either\nthe calculated delay or the maximum delay specified. This creates an upper bound\non the maximum delay we can see.\n\nIn order to find the actual delay value we apply additive noise which is proportional\nto the current desired delay. This ensures that the actual delay is kept within\nthe expected order of magnitude, while still having some randomness, which ensures\nthat our retried requests don't \"harmonize\" making it harder for the downstream service to heal.","ref":"Tesla.Middleware.Retry.html#module-backoff-algorithm","title":"Backoff algorithm - Tesla.Middleware.Retry","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.Retry,\n    delay: 500,\n    max_retries: 10,\n    max_delay: 4_000,\n    should_retry: fn\n      {:ok, %{status: status}} when status in [400, 500] -> true\n      {:ok, _} -> false\n      {:error, _} -> true\n    end\n    # or\n    plug Tesla.Middleware.Retry, should_retry: fn\n      {:ok, %{status: status}}, _env, _context when status in [400, 500] -> true\n      {:ok, _reason}, _env, _context -> false\n      {:error, _reason}, %Tesla.Env{method: :post}, _context -> false\n      {:error, _reason}, %Tesla.Env{method: :put}, %{retries: 2} -> false\n      {:error, _reason}, _env, _context -> true\n    end\nend\n```","ref":"Tesla.Middleware.Retry.html#module-examples","title":"Examples - Tesla.Middleware.Retry","type":"module"},{"doc":"- `:delay` - The base delay in milliseconds (positive integer, defaults to 50)\n- `:max_retries` - maximum number of retries (non-negative integer, defaults to 5)\n- `:max_delay` - maximum delay in milliseconds (positive integer, defaults to 5000)\n- `:should_retry` - function with an arity of 1 or 3 used to determine if the request should\n    be retried the first argument is the result, the second is the env and the third is\n    the context: options + `:retries` (defaults to a match on `{:error, _reason}`)\n- `:jitter_factor` - additive noise proportionality constant\n    (float between 0 and 1, defaults to 0.2)","ref":"Tesla.Middleware.Retry.html#module-options","title":"Options - Tesla.Middleware.Retry","type":"module"},{"doc":"Decode Server Sent Events.\n\nThis middleware is mostly useful when streaming response body.","ref":"Tesla.Middleware.SSE.html","title":"Tesla.Middleware.SSE","type":"module"},{"doc":"```\nplug Tesla.Middleware.SSE, only: :data\n\n```","ref":"Tesla.Middleware.SSE.html#module-examples","title":"Examples - Tesla.Middleware.SSE","type":"module"},{"doc":"- `:only` - keep only specified keys in event (necessary for using with `JSON` middleware)\n- `:decode_content_types` - list of additional decodable content-types","ref":"Tesla.Middleware.SSE.html#module-options","title":"Options - Tesla.Middleware.SSE","type":"module"},{"doc":"","ref":"Tesla.Middleware.SSE.html#decode/2","title":"Tesla.Middleware.SSE.decode/2","type":"function"},{"doc":"Emits events using the `:telemetry` library to expose instrumentation.","ref":"Tesla.Middleware.Telemetry.html","title":"Tesla.Middleware.Telemetry","type":"module"},{"doc":"```\ndefmodule MyClient do\n  use Tesla\n\n  plug Tesla.Middleware.Telemetry\nend\n\n:telemetry.attach(\n  \"my-tesla-telemetry\",\n  [:tesla, :request, :stop],\n  fn event, measurements, meta, config ->\n    # Do something with the event\n  end,\n  nil\n)\n```","ref":"Tesla.Middleware.Telemetry.html#module-examples","title":"Examples - Tesla.Middleware.Telemetry","type":"module"},{"doc":"- `:metadata` - additional metadata passed to telemetry events","ref":"Tesla.Middleware.Telemetry.html#module-options","title":"Options - Tesla.Middleware.Telemetry","type":"module"},{"doc":"* `[:tesla, :request, :start]` - emitted at the beginning of the request.\n  * Measurement: `%{system_time: System.system_time()}`\n  * Metadata: `%{env: Tesla.Env.t()}`\n\n* `[:tesla, :request, :stop]` - emitted at the end of the request.\n  * Measurement: `%{duration: native_time}`\n  * Metadata: `%{env: Tesla.Env.t()} | %{env: Tesla.Env.t(), error: term()}`\n\n* `[:tesla, :request, :exception]` - emitted when an exception has been raised.\n  * Measurement: `%{duration: native_time}`\n  * Metadata: `%{env: Tesla.Env.t(), kind: Exception.kind(), reason: term(), stacktrace: Exception.stacktrace()}`","ref":"Tesla.Middleware.Telemetry.html#module-telemetry-events","title":"Telemetry Events - Tesla.Middleware.Telemetry","type":"module"},{"doc":"* `[:tesla, :request]` - This event is emitted for backwards compatibility only and should be considered deprecated.\n      This event can be disabled by setting `config :tesla, Tesla.Middleware.Telemetry, disable_legacy_event: true` in your config.\n      Be sure to run `mix deps.compile --force tesla` after changing this setting to ensure the change is picked up.\n\nPlease check the [telemetry](https://hexdocs.pm/telemetry/) for the further usage.","ref":"Tesla.Middleware.Telemetry.html#module-legacy-telemetry-events","title":"Legacy Telemetry Events - Tesla.Middleware.Telemetry","type":"module"},{"doc":"Sometimes, it is useful to have access to a template url (i.e. `\"/users/:user_id\"`) for grouping\nTelemetry events. For such cases, a combination of the `Tesla.Middleware.PathParams`,\n`Tesla.Middleware.Telemetry` and `Tesla.Middleware.KeepRequest` may be used.\n\n```\ndefmodule MyClient do\n  use Tesla\n\n  # The KeepRequest middleware sets the template url as a Tesla.Env.opts entry\n  # Said entry must be used because on happy-path scenarios,\n  # the Telemetry middleware will receive the Tesla.Env.url resolved by PathParams.\n  plug Tesla.Middleware.KeepRequest\n  plug Tesla.Middleware.Telemetry\n  plug Tesla.Middleware.PathParams\nend\n\n:telemetry.attach(\n  \"my-tesla-telemetry\",\n  [:tesla, :request, :stop],\n  fn event, measurements, meta, config ->\n    path_params_template_url = meta.env.opts[:req_url]\n    # The meta.env.url key will only present the resolved URL on happy-path scenarios.\n    # Error cases will still return the original template url.\n    path_params_resolved_url = meta.env.url\n  end,\n  nil\n)\n```","ref":"Tesla.Middleware.Telemetry.html#module-url-event-scoping-with-tesla-middleware-pathparams-and-tesla-middleware-keeprequest","title":"URL event scoping with `Tesla.Middleware.PathParams` and `Tesla.Middleware.KeepRequest` - Tesla.Middleware.Telemetry","type":"module"},{"doc":"Timeout HTTP request after X milliseconds.","ref":"Tesla.Middleware.Timeout.html","title":"Tesla.Middleware.Timeout","type":"module"},{"doc":"defmodule MyClient do\n      use Tesla\n\n      plug Tesla.Middleware.Timeout, timeout: 2_000\n    end\n\nIf you are using OpenTelemetry in your project, you may be interested in\nusing `OpentelemetryProcessPropagator.Task` to have a better integration using\nthe `task_module` option.\n\n    defmodule MyClient do\n      use Tesla\n\n      plug Tesla.Middleware.Timeout,\n        timeout: 2_000,\n        task_module: OpentelemetryProcessPropagator.Task\n    end","ref":"Tesla.Middleware.Timeout.html#module-examples","title":"Examples - Tesla.Middleware.Timeout","type":"module"},{"doc":"- `:timeout` - number of milliseconds a request is allowed to take (defaults to `1000`)\n- `:task_module` - the `Task` module used to spawn tasks. Useful when you want\n  use alternatives such as `OpentelemetryProcessPropagator.Task` from OTEL\n  project.","ref":"Tesla.Middleware.Timeout.html#module-options","title":"Options - Tesla.Middleware.Timeout","type":"module"},{"doc":"","ref":"Tesla.Error.html","title":"Tesla.Error","type":"exception"},{"doc":"","ref":"Tesla.Error.html#message/1","title":"Tesla.Error.message/1","type":"function"},{"doc":"","ref":"Tesla.Mock.Error.html","title":"Tesla.Mock.Error","type":"exception"},{"doc":"","ref":"Tesla.Mock.Error.html#message/1","title":"Tesla.Mock.Error.message/1","type":"function"},{"doc":"# Tesla\n\n[![Build Status](https://github.com/teamon/tesla/workflows/Test/badge.svg?branch=master)](https://github.com/teamon/tesla/actions)\n[![Hex.pm](https://img.shields.io/hexpm/v/tesla.svg)](https://hex.pm/packages/tesla)\n[![Hexdocs.pm](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/tesla/)\n[![Hex.pm](https://img.shields.io/hexpm/dt/tesla.svg)](https://hex.pm/packages/tesla)\n[![Hex.pm](https://img.shields.io/hexpm/dw/tesla.svg)](https://hex.pm/packages/tesla)\n[![codecov](https://codecov.io/gh/teamon/tesla/branch/master/graph/badge.svg)](https://codecov.io/gh/teamon/tesla)\n[![Inline docs](https://inch-ci.org/github/teamon/tesla.svg)](https://inch-ci.org/github/teamon/tesla)\n\nTesla is an HTTP client loosely based on [Faraday](https://github.com/lostisland/faraday).\nIt embraces the concept of middleware when processing the request/response cycle.\n\n> Note that this README refers to the `master` branch of Tesla, not the latest\n> released version on Hex. See [the documentation](https://hexdocs.pm/tesla) for\n> the documentation of the version you're using.\n\nFor the list of changes, checkout the latest [release notes](https://github.com/teamon/tesla/releases).\n\n---","ref":"readme.html","title":"Tesla","type":"extras"},{"doc":"Define module with `use Tesla` and choose from a variety of middleware.\n\n```elixir\ndefmodule GitHub do\n  use Tesla\n\n  plug Tesla.Middleware.BaseUrl, \"https://api.github.com\"\n  plug Tesla.Middleware.Headers, [{\"authorization\", \"token xyz\"}]\n  plug Tesla.Middleware.JSON\n\n  def user_repos(login) do\n    get(\"/users/\" <> login <> \"/repos\")\n  end\nend\n```\n\nThen use it like this:\n\n```elixir\n{:ok, response} = GitHub.user_repos(\"teamon\")\n\nresponse.status\n# => 200\n\nresponse.body\n# => [%{…}, …]\n\nresponse.headers\n# => [{\"content-type\", \"application/json\"}, ...]\n```\n\nSee below for documentation.","ref":"readme.html#http-client-example","title":"HTTP Client example - Tesla","type":"extras"},{"doc":"Add `:tesla` as dependency in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:tesla, \"~> 1.9\"},\n\n    # optional, but recommended adapter\n    {:hackney, \"~> 1.20\"},\n\n    # optional, required by JSON middleware\n    {:jason, \"~> 1.4\"}\n  ]\nend\n```\n\nTesla uses [Semantic Versioning](https://semver.org) 2.0.\n\nConfigure default adapter in `config/config.exs` (optional).\n\n```elixir\n# config/config.exs\n\nconfig :tesla, adapter: Tesla.Adapter.Hackney\n```\n\n> The default adapter is erlang's built-in `httpc`, but it is not recommended\n> to use it in production environment as it does not validate SSL certificates\n> [among other issues](https://github.com/teamon/tesla/issues?utf8=%E2%9C%93&q=is%3Aissue+label%3Ahttpc+).","ref":"readme.html#installation","title":"Installation - Tesla","type":"extras"},{"doc":"- [Middleware](#middleware)\n- [Runtime middleware](#runtime-middleware)\n- [Adapters](#adapters)\n- [Streaming](#streaming)\n- [Multipart](#multipart)\n- [Testing](#testing)\n- [Writing middleware](#writing-middleware)\n- [Direct usage](#direct-usage)\n- [Cheatsheet](#cheatsheet)\n- [Cookbook](https://github.com/teamon/tesla/wiki)\n- [Changelog](https://github.com/teamon/tesla/releases)","ref":"readme.html#documentation","title":"Documentation - Tesla","type":"extras"},{"doc":"Tesla is built around the concept of composable middlewares.\nThis is very similar to how [Plug Router](https://github.com/elixir-plug/plug#plugrouter) works.","ref":"readme.html#middleware","title":"Middleware - Tesla","type":"extras"},{"doc":"- [`Tesla.Middleware.BaseUrl`](https://hexdocs.pm/tesla/Tesla.Middleware.BaseUrl.html) - set base url\n- [`Tesla.Middleware.Headers`](https://hexdocs.pm/tesla/Tesla.Middleware.Headers.html) - set request headers\n- [`Tesla.Middleware.Query`](https://hexdocs.pm/tesla/Tesla.Middleware.Query.html) - set query parameters\n- [`Tesla.Middleware.Opts`](https://hexdocs.pm/tesla/Tesla.Middleware.Opts.html) - set request options\n- [`Tesla.Middleware.FollowRedirects`](https://hexdocs.pm/tesla/Tesla.Middleware.FollowRedirects.html) - follow HTTP 3xx redirects\n- [`Tesla.Middleware.MethodOverride`](https://hexdocs.pm/tesla/Tesla.Middleware.MethodOverride.html) - set `X-Http-Method-Override` header\n- [`Tesla.Middleware.Logger`](https://hexdocs.pm/tesla/Tesla.Middleware.Logger.html) - log requests (method, url, status, and time)\n- [`Tesla.Middleware.KeepRequest`](https://hexdocs.pm/tesla/Tesla.Middleware.KeepRequest.html) - keep request `body` and `headers`\n- [`Tesla.Middleware.PathParams`](https://hexdocs.pm/tesla/Tesla.Middleware.PathParams.html) - use templated URLs","ref":"readme.html#basic","title":"Basic - Tesla","type":"extras"},{"doc":"- [`Tesla.Middleware.FormUrlencoded`](https://hexdocs.pm/tesla/Tesla.Middleware.FormUrlencoded.html) - urlencode POST body, useful for POSTing a map/keyword list\n- [`Tesla.Middleware.JSON`](https://hexdocs.pm/tesla/Tesla.Middleware.JSON.html) - JSON request/response body\n- [`Tesla.Middleware.Compression`](https://hexdocs.pm/tesla/Tesla.Middleware.Compression.html) - `gzip` and `deflate`\n- [`Tesla.Middleware.DecodeRels`](https://hexdocs.pm/tesla/Tesla.Middleware.DecodeRels.html) - decode `Link` header into `opts[:rels]` field in response","ref":"readme.html#formats","title":"Formats - Tesla","type":"extras"},{"doc":"- [`Tesla.Middleware.BasicAuth`](https://hexdocs.pm/tesla/Tesla.Middleware.BasicAuth.html) - HTTP Basic Auth\n- [`Tesla.Middleware.BearerAuth`](https://hexdocs.pm/tesla/Tesla.Middleware.BearerAuth.html) - HTTP Bearer Auth\n- [`Tesla.Middleware.DigestAuth`](https://hexdocs.pm/tesla/Tesla.Middleware.DigestAuth.html) - Digest access authentication","ref":"readme.html#auth","title":"Auth - Tesla","type":"extras"},{"doc":"- [`Tesla.Middleware.Timeout`](https://hexdocs.pm/tesla/Tesla.Middleware.Timeout.html) - timeout request after X milliseconds despite of server response\n- [`Tesla.Middleware.Retry`](https://hexdocs.pm/tesla/Tesla.Middleware.Retry.html) - retry few times in case of connection refused\n- [`Tesla.Middleware.Fuse`](https://hexdocs.pm/tesla/Tesla.Middleware.Fuse.html) - fuse circuit breaker integration","ref":"readme.html#error-handling","title":"Error handling - Tesla","type":"extras"},{"doc":"All HTTP functions, such as `Tesla.get/3` and `Tesla.post/4`, can take a dynamic client as the first argument.\nThis allows to use convenient syntax for modifying the behaviour in runtime.\n\nConsider the following case: GitHub API can be accessed using OAuth token authorization.\n\nWe can't use `plug Tesla.Middleware.Headers, [{\"authorization\", \"token here\"}]`\nsince this would be compiled only once and there is no way to insert dynamic user token.\n\nInstead, we can use `Tesla.client` to create a client with dynamic middleware:\n\n```elixir\ndefmodule GitHub do\n  # notice there is no `use Tesla`\n\n  def user_repos(client, login) do\n    # pass `client` argument to `Tesla.get` function\n    Tesla.get(client, \"/user/\" <> login <> \"/repos\")\n  end\n\n  def issues(client) do\n    Tesla.get(client, \"/issues\")\n  end\n\n  # build dynamic client based on runtime arguments\n  def client(token) do\n    middleware = [\n      {Tesla.Middleware.BaseUrl, \"https://api.github.com\"},\n      Tesla.Middleware.JSON,\n      {Tesla.Middleware.Headers, [{\"authorization\", \"token: \" <> token }]}\n    ]\n\n    Tesla.client(middleware)\n  end\nend\n```\n\nand then:\n\n```elixir\nclient = GitHub.client(user_token)\nclient |> GitHub.user_repos(\"teamon\")\nclient |> GitHub.get(\"/me\")\n```","ref":"readme.html#runtime-middleware","title":"Runtime middleware - Tesla","type":"extras"},{"doc":"Tesla supports multiple HTTP adapter that do the actual HTTP request processing.\n\n- [`Tesla.Adapter.Httpc`](https://hexdocs.pm/tesla/Tesla.Adapter.Httpc.html) - the default, built-in erlang [httpc](https://erlang.org/doc/man/httpc.html) adapter\n- [`Tesla.Adapter.Hackney`](https://hexdocs.pm/tesla/Tesla.Adapter.Hackney.html) - [hackney](https://github.com/benoitc/hackney), \"simple HTTP client in Erlang\"\n- [`Tesla.Adapter.Ibrowse`](https://hexdocs.pm/tesla/Tesla.Adapter.Ibrowse.html) - [ibrowse](https://github.com/cmullaparthi/ibrowse), \"Erlang HTTP client\"\n- [`Tesla.Adapter.Gun`](https://hexdocs.pm/tesla/Tesla.Adapter.Gun.html) - [gun](https://github.com/ninenines/gun), \"HTTP/1.1, HTTP/2 and Websocket client for Erlang/OTP\"\n- [`Tesla.Adapter.Mint`](https://hexdocs.pm/tesla/Tesla.Adapter.Mint.html) - [mint](https://github.com/elixir-mint/mint), \"Functional HTTP client for Elixir with support for HTTP/1 and HTTP/2\"\n- [`Tesla.Adapter.Finch`](https://hexdocs.pm/tesla/Tesla.Adapter.Finch.html) - [finch](https://github.com/keathley/finch), \"An HTTP client with a focus on performance, built on top of [Mint](https://github.com/elixir-mint/mint) and [NimblePool](https://github.com/dashbitco/nimble_pool).\"\n\nWhen using adapter other than `:httpc` remember to add it to the dependencies list in `mix.exs`\n\n```elixir\ndefp deps do\n  [\n    {:tesla, \"~> 1.9\"},\n    {:hackney, \"~> 1.20\"} # when using hackney adapter\n  ]\nend\n```","ref":"readme.html#adapters","title":"Adapters - Tesla","type":"extras"},{"doc":"In case there is a need to pass specific adapter options you can do it in one of four ways:\n\nSupplying them as a keyword list in a tuple via config:\n\n```elixir\nconfig :tesla, adapter: {Tesla.Adapter.Hackney, [recv_timeout: 30_000]}\n```\n\nUsing `adapter` macro:\n\n```elixir\ndefmodule GitHub do\n  use Tesla\n\n  adapter Tesla.Adapter.Hackney, recv_timeout: 30_000, ssl_options: [certfile: \"certs/client.crt\"]\nend\n```\n\nUsing `Tesla.client/2`:\n\n```elixir\ndef new(...) do\n  middleware = [...]\n  adapter = {Tesla.Adapter.Hackney, [recv_timeout: 30_000]}\n  Tesla.client(middleware, adapter)\nend\n```\n\nPassing directly to request functions such as `MyClient.get/3` or `Tesla.get/3`.\n\n```elixir\nMyClient.get(\"/\", opts: [adapter: [recv_timeout: 30_000]])\nTesla.get(client, \"/\", opts: [adapter: [recv_timeout: 30_000]])\n```","ref":"readme.html#adapter-options","title":"Adapter options - Tesla","type":"extras"},{"doc":"","ref":"readme.html#streaming","title":"Streaming - Tesla","type":"extras"},{"doc":"If adapter supports it, you can pass a\n[Stream](https://hexdocs.pm/elixir/main/Stream.html) as request\nbody, e.g.:\n\n```elixir\ndefmodule ElasticSearch do\n  use Tesla\n\n  plug Tesla.Middleware.BaseUrl, \"http://localhost:9200\"\n  plug Tesla.Middleware.JSON\n\n  def index(records_stream) do\n    stream = records_stream |> Stream.map(fn record -> %{index: [some, data]} end)\n    post(\"/_bulk\", stream)\n  end\nend\n```\n\nEach piece of stream will be encoded as JSON and sent as a new line (conforming\nto JSON stream format).","ref":"readme.html#streaming-request-body","title":"Streaming Request Body - Tesla","type":"extras"},{"doc":"If adapter supports it, you can pass a `response: :stream` option to return\nresponse body as a\n[Stream](https://elixir-lang.org/docs/stable/elixir/Stream.html)\n\n```elixir\ndefmodule OpenAI do\n  def new(token) do\n    middleware = [\n      {Tesla.Middleware.BaseUrl, \"https://api.openai.com/v1\"},\n      {Tesla.Middleware.BearerAuth, token: token},\n      {Tesla.Middleware.JSON, decode_content_types: [\"text/event-stream\"]},\n      {Tesla.Middleware.SSE, only: :data}\n    ]\n    Tesla.client(middleware, {Tesla.Adapter.Finch, name: MyFinch})\n  end\n\n  def completion(client, prompt) do\n    data = %{\n      model: \"gpt-3.5-turbo\",\n      messages: [%{role: \"user\", content: prompt}],\n      stream: true\n    }\n    Tesla.post(client, \"/chat/completions\", data, opts: [adapter: [response: :stream]])\n  end\nend\nclient = OpenAI.new(\" \")\n{:ok, env} = OpenAI.completion(client, \"What is the meaning of life?\")\nenv.body\n|> Stream.each(fn chunk -> IO.inspect(chunk) end)\n```","ref":"readme.html#streaming-response-body","title":"Streaming Response Body - Tesla","type":"extras"},{"doc":"You can pass a `Tesla.Multipart` struct as the body:\n\n```elixir\nalias Tesla.Multipart\n\nmp =\n  Multipart.new()\n  |> Multipart.add_content_type_param(\"charset=utf-8\")\n  |> Multipart.add_field(\"field1\", \"foo\")\n  |> Multipart.add_field(\"field2\", \"bar\",\n    headers: [{\"content-id\", \"1\"}, {\"content-type\", \"text/plain\"}]\n  )\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\")\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\", name: \"foobar\")\n  |> Multipart.add_file_content(\"sample file content\", \"sample.txt\")\n\n{:ok, response} = MyApiClient.post(\"https://httpbin.org/post\", mp)\n```","ref":"readme.html#multipart","title":"Multipart - Tesla","type":"extras"},{"doc":"You can set the adapter to `Tesla.Mock` in tests:\n\n```elixir\n# config/test.exs\n# Use mock adapter for all clients\nconfig :tesla, adapter: Tesla.Mock\n# or only for one\nconfig :tesla, MyApi, adapter: Tesla.Mock\n```\n\nThen, mock requests before using your client:\n\n```elixir\ndefmodule MyAppTest do\n  use ExUnit.Case\n\n  import Tesla.Mock\n\n  setup do\n    mock(fn\n      %{method: :get, url: \"https://example.com/hello\"} ->\n        %Tesla.Env{status: 200, body: \"hello\"}\n\n      %{method: :post, url: \"https://example.com/world\"} ->\n        json(%{\"my\" => \"data\"})\n    end)\n\n    :ok\n  end\n\n  test \"list things\" do\n    assert {:ok, %Tesla.Env{} = env} = MyApi.get(\"https://example.com/hello\")\n    assert env.status == 200\n    assert env.body == \"hello\"\n  end\nend\n```","ref":"readme.html#testing","title":"Testing - Tesla","type":"extras"},{"doc":"A Tesla middleware is a module with `c:Tesla.Middleware.call/3` function, that at some point calls `Tesla.run/2` with `env` and `next` to process the rest of stack.\n\n```elixir\ndefmodule MyMiddleware do\n  @behaviour Tesla.Middleware\n\n  def call(env, next, options) do\n    env\n    |> do_something_with_request()\n    |> Tesla.run(next)\n    |> do_something_with_response()\n  end\nend\n```\n\nThe arguments are:\n\n- `env` - `Tesla.Env` instance\n- `next` - middleware continuation stack; to be executed with `Tesla.run/2` with `env` and `next`\n- `options` - arguments passed during middleware configuration (`plug MyMiddleware, options`)\n\nThere is no distinction between request and response middleware, it's all about executing `Tesla.run/2` function at the correct time.\n\nFor example, a request logger middleware could be implemented like this:\n\n```elixir\ndefmodule Tesla.Middleware.RequestLogger do\n  @behaviour Tesla.Middleware\n\n  def call(env, next, _) do\n    env\n    |> IO.inspect()\n    |> Tesla.run(next)\n  end\nend\n```\n\nand response logger middleware like this:\n\n```elixir\ndefmodule Tesla.Middleware.ResponseLogger do\n  @behaviour Tesla.Middleware\n\n  def call(env, next, _) do\n    env\n    |> Tesla.run(next)\n    |> IO.inspect()\n  end\nend\n```\n\nSee [built-in middlewares](https://github.com/teamon/tesla/tree/master/lib/tesla/middleware) for more examples.\n\nMiddleware should have documentation following this template:\n\n````elixir\ndefmodule Tesla.Middleware.SomeMiddleware do\n  @moduledoc \"\"\"\n  Short description what it does\n\n  Longer description, including e.g. additional dependencies.","ref":"readme.html#writing-middleware","title":"Writing middleware - Tesla","type":"extras"},{"doc":"```elixir\n  defmodule MyClient do\n    use Tesla\n\n    plug Tesla.Middleware.SomeMiddleware, most: :common, options: \"here\"\n  end\n  ```","ref":"readme.html#examples","title":"Examples - Tesla","type":"extras"},{"doc":"- `:list` - all possible options\n  - `:with` - their default values\n  \"\"\"\n\n  @behaviour Tesla.Middleware\nend\n````","ref":"readme.html#options","title":"Options - Tesla","type":"extras"},{"doc":"You can also use Tesla directly, without creating a client module.\nThis however won’t include any middleware.\n\n```elixir\n# Example get request\n{:ok, response} = Tesla.get(\"https://httpbin.org/ip\")\n\nresponse.status\n# => 200\n\nresponse.body\n# => \"{\\n  \"origin\": \"87.205.72.203\"\\n}\\n\"\n\nresponse.headers\n# => [{\"content-type\", \"application/json\" ...}]\n\n{:ok, response} = Tesla.get(\"https://httpbin.org/get\", query: [a: 1, b: \"foo\"])\n\n# Example post request\n{:ok, response} =\n  Tesla.post(\"https://httpbin.org/post\", \"data\", headers: [{\"content-type\", \"application/json\"}])\n```","ref":"readme.html#direct-usage","title":"Direct usage - Tesla","type":"extras"},{"doc":"","ref":"readme.html#cheatsheet","title":"Cheatsheet - Tesla","type":"extras"},{"doc":"```elixir\n# GET /path\nget(\"/path\")\n\n# GET /path?a=hi&b[]=1&b[]=2&b[]=3\nget(\"/path\", query: [a: \"hi\", b: [1, 2, 3]])\n\n# GET with dynamic client\nget(client, \"/path\")\nget(client, \"/path\", query: [page: 3])\n\n# arguments are the same for GET, HEAD, OPTIONS & TRACE\nhead(\"/path\")\noptions(\"/path\")\ntrace(\"/path\")\n\n# POST, PUT, PATCH\npost(\"/path\", \"some-body-i-used-to-know\")\nput(\"/path\", \"some-body-i-used-to-know\", query: [a: \"0\"])\npatch(\"/path\", multipart)\n```","ref":"readme.html#making-requests-101","title":"Making requests 101 - Tesla","type":"extras"},{"doc":"```elixir\n# generate only get and post function\nuse Tesla, only: ~w(get post)a\n\n# generate only delete function\nuse Tesla, only: [:delete]\n\n# generate all functions except delete and options\nuse Tesla, except: [:delete, :options]\n```","ref":"readme.html#configuring-http-functions-visibility","title":"Configuring HTTP functions visibility - Tesla","type":"extras"},{"doc":"```elixir\nuse Tesla, docs: false\n```","ref":"readme.html#disable-docs-for-http-functions","title":"Disable docs for HTTP functions - Tesla","type":"extras"},{"doc":"```elixir\nplug Tesla.Middleware.EncodeJson\n```","ref":"readme.html#encode-only-json-request-do-not-decode-response","title":"Encode only JSON request (do not decode response) - Tesla","type":"extras"},{"doc":"```elixir\nplug Tesla.Middleware.DecodeJson\n```","ref":"readme.html#decode-only-json-response-do-not-encode-request","title":"Decode only JSON response (do not encode request) - Tesla","type":"extras"},{"doc":"```elixir\n# use JSX\nplug Tesla.Middleware.JSON, engine: JSX, engine_opts: [strict: [:comments]]\n\n# use custom functions\nplug Tesla.Middleware.JSON, decode: &JSX.decode/1, encode: &JSX.encode/1\n```","ref":"readme.html#use-other-json-library","title":"Use other JSON library - Tesla","type":"extras"},{"doc":"```elixir\ndefmodule Tesla.Middleware.MyCustomMiddleware do\n  def call(env, next, options) do\n    env\n    |> do_something_with_request()\n    |> Tesla.run(next)\n    |> do_something_with_response()\n  end\nend\n```\n\n## [`0.x` to `1.0` Migration Guide](https://github.com/teamon/tesla/wiki/0.x-to-1.0-Migration-Guide)\n\n[Documentation for 0.x branch](https://github.com/teamon/tesla/tree/0.x)","ref":"readme.html#custom-middleware","title":"Custom middleware - Tesla","type":"extras"},{"doc":"1. Fork it (https://github.com/teamon/tesla/fork)\n2. Create your feature branch (`git checkout -b my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin my-new-feature`)\n5. Create new Pull Request","ref":"readme.html#contributing","title":"Contributing - Tesla","type":"extras"},{"doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details\n\nCopyright (c) 2015-2021 [Tymon Tobolski](https://teamon.me/about/)\n\n---","ref":"readme.html#license","title":"License - Tesla","type":"extras"},{"doc":"This project is sponsored by [ubots - Useful bots for Slack](https://ubots.xyz/)","ref":"readme.html#sponsors","title":"Sponsors - Tesla","type":"extras"},{"doc":"The MIT License (MIT)\n\nCopyright (c) 2015-2021 Tymon Tobolski\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}